import { Command } from 'commander';

declare function buildPromptCommands(program: Command): void;

declare function buildPostCommands(program: Command): void;

declare function buildPostTemplateCommands(program: Command): void;

declare function buildWpCommands(program: Command): void;

type Wordpress = {
    domain: string;
    username: string;
    password: string;
};

declare function initStore(): void;
declare function getAllWordpress(): Promise<Wordpress[]>;
declare function addWordpress(wp: Wordpress): Promise<void>;
declare function getWordpress(domain: string): Promise<Wordpress | undefined>;
declare function removeWordpress(domain: string): Promise<Boolean>;
declare function exportWordpressList(exportFile: string): Promise<void>;
declare function importWordpressList(importFile: string): Promise<void>;

type llm = 'gpt-4' | 'gpt-4-turbo-preview' | 'mistral-small-latest' | 'mistral-medium-latest' | 'mistral-large-latest' | 'claude' | 'groq';
declare const DEFAULT_LLM: llm;
declare function getLLMs(): llm[];
type BasePostPrompt = {
    model: llm;
    temperature?: number;
    frequencyPenalty?: number;
    presencePenalty?: number;
    debug?: boolean;
    debugapi?: boolean;
    apiKey?: string;
    filename?: string;
    promptFolder?: string;
};
type AutoPostPrompt = BasePostPrompt & {
    topic?: string;
    country?: string;
    intent?: string;
    audience?: string;
    language: string;
    generate?: boolean;
    withConclusion?: boolean;
};
type TemplatePostPrompt = BasePostPrompt & {
    templateFile: string;
    input: any;
};
type TemplatePrompt = {
    type: 's' | 'c' | 'i';
    prompt: string;
};
type Heading = {
    title: string;
    keywords?: string[];
    headings?: Heading[];
};
type PostOutline = {
    title: string;
    headings: Heading[];
    slug: string;
    seoTitle: string;
    seoDescription: string;
};
type Post = {
    h1: string;
    content: string;
    seoTitle: string;
    seoDescription: string;
    slug: string;
    categories?: number[];
    status?: string;
};
type TemplatePost = {
    h1: string;
    content: string;
    seoTitle: string;
    seoDescription: string;
    slug: string;
    categories?: number[];
    status?: string;
};
type SeoInfo = {
    h1: string;
    slug: string;
    seoTitle: string;
    seoDescription: string;
};
type AudienceIntentInfo = {
    audience: string;
    intent: string;
};

declare class PostGenerator {
    private postPrompt;
    private llm_json;
    private llm_content;
    private memory;
    private log;
    private promptFolder;
    constructor(postPrompt: AutoPostPrompt);
    generate(): Promise<Post>;
    /**
     * Generate the audience and intent.
     */
    generateAudienceAndIntent(): Promise<{
        audience: string;
        intent: string;
    }>;
    /**
     * Generate a post outline.
     */
    private generateOutline;
    private generateIntroduction;
    private generateConclusion;
    private generateHeadingContents;
    private buildContent;
    private generateHeadingContent;
    /**
     *
     * Generate a content in markdown format based on a langchain template
     * Mainly used for the introduction and conclusion
     *
     */
    private generateContent;
    /**
     * Convert a post prompt to a string for adding to the memory.
     */
    private promptToString;
    /**
     * Convert a post outline to a markdown string for adding to the memory.
     */
    private postOutlineToMarkdown;
    private debugMemory;
}
declare class PostTemplateGenerator {
    private postPrompt;
    private llm_content;
    private llm_json;
    private memory;
    private log;
    private promptFolder;
    constructor(postPrompt: TemplatePostPrompt);
    generate(): Promise<TemplatePost>;
    private generateTemplateContent;
    private generateSeoInfo;
    private readTemplate;
}

export { type AudienceIntentInfo, type AutoPostPrompt, type BasePostPrompt, DEFAULT_LLM, type Heading, type Post, PostGenerator, type PostOutline, PostTemplateGenerator, type SeoInfo, type TemplatePost, type TemplatePostPrompt, type TemplatePrompt, addWordpress, buildPostCommands, buildPostTemplateCommands, buildPromptCommands, buildWpCommands, exportWordpressList, getAllWordpress, getLLMs, getWordpress, importWordpressList, initStore, type llm, removeWordpress };
